@@ -1,4 +1,4 @@
-static char *rcsid="$Id: cgiwrap.c,v 1.31 1998/07/31 23:47:01 nneul Exp $";
+static char *rcsid="$Id: cgiwrap.c,v 1.32 1999/04/14 20:15:14 nneul Exp $";
 
 /**
  **  File: cgiwrap.c
@@ -60,7 +60,10 @@ int main (int argc, char *argv[])
 	/* Redirect stderr to stdout */
 #if defined(CONF_REDIR_STDERR)
 	DEBUG_Msg("Redirecting STDERR to STDOUT\n");
-	dup2(1,2);
+	if (dup2(1,2) == -1) 
+	{
+		MSG_Error_SystemError("Call to dup2() to redirect stderr to stdout failed!");
+	}
 #endif
 
 	/* Check who is running this script */
