@@ -1,4 +1,4 @@
-/* rcsid: $Id: msgs.h,v 1.7 1998/01/25 04:15:44 nneul Exp $ */
+/* rcsid: $Id: msgs.h,v 1.8 1999/04/17 17:36:18 nneul Exp $ */
 
 /**
  **  File: messages.h
@@ -19,7 +19,7 @@ void MSG_Error_General(char *message);
 void MSG_Error_ServerUser(char *msg);
 void MSG_Error_ExecutionNotPermitted(char *path, char *reason);
 void MSG_Error_SystemError(char *when);
-void MSG_Error_AccessControl(char *why);
+void MSG_Error_AccessControl(char *why, char *controlfile);
 void MSG_Error_ExecFailed(void);
 void MSG_Error_NoSuchUser(char *user);
 void MSG_Error_NoScriptDir(void);
