@@ -1,4 +1,4 @@
-static char *rcsid="$Id: stdutil.c,v 1.11 1997/11/16 21:54:53 nneul Exp $";
+static char *rcsid="$Id: stdutil.c,v 1.12 1999/04/14 20:15:14 nneul Exp $";
 
 /**
  **  File: stdutil.c
@@ -15,11 +15,7 @@ char *strdup(char *str)
 {
 	char *temp;
 
-	temp = (char *) malloc ( strlen(str) + 1 );
-	if (!temp)
-	{
-		MSG_Error_SystemError("Couldn't malloc memory for string!");
-	}
+	temp = (char *) SafeMalloc ( strlen(str) + 1, "strdup" );
 	strcpy(temp,str);
 
 	return temp;
