@@ -1,4 +1,4 @@
-/* rcsid: $Id: util.h,v 1.15 1998/01/13 01:48:02 nneul Exp $ */
+/* rcsid: $Id: util.h,v 1.16 1999/04/14 20:15:14 nneul Exp $ */
 
 /**
  **  File: util.h
@@ -6,8 +6,8 @@
  **/ 
 
 char *SafeMalloc( size_t size, char *what );
+void SafePutenv( char * string, char *what );
 char **CreateARGV( char *scrStr, int argc, char *argv[]);
-char *StripPrefix(char *, char *);
 void ChangeID(struct passwd *user);
 void ChangeAuxGroups(struct passwd *user);
 void SetResourceLimits(void);
